=== modified file 'bzrlib/tests/test_sftp_transport.py'
--- bzrlib/tests/test_sftp_transport.py	2006-07-20 23:55:43 +0000
+++ bzrlib/tests/test_sftp_transport.py	2006-08-01 10:53:24 +0000
@@ -15,7 +15,7 @@
 # Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 
 import os
-from operator import lt, gt
+from operator import lt, gt, ge
 import socket
 import threading
 import time
@@ -243,13 +243,82 @@
         # change the latency knob to 100ms. We take about 40ms for a 
         # loopback connection ordinarily.
         self.get_server().add_latency = 0.1
-        self.assertConnectionTime(gt)
+        self.assertConnectionTime(ge)
 
     def assertConnectionTime(self, operator):
-        start_time = time.time()
+        from bzrlib.transport.sftp import SocketDelay
+        start_time = SocketDelay.simulated_time
         transport = self.get_transport()
-        stop_time = time.time()
+        stop_time = SocketDelay.simulated_time
         self.failUnless(operator(stop_time - start_time, 0.1))
 
     def test_default_fast(self):
         self.assertConnectionTime(lt)
+
+class FakeSocket(object):
+    """ Fake socket object used to test the SocketDelay wrapper without
+    using a real socket."""
+
+    def __init__(self):
+        self._data = ""
+
+    def send(self, data, flags=0):
+        self._data += data
+        return len(data)
+
+    def sendall(self, data, flags=0):
+        self._data += data
+        return len(data)
+
+    def recv(self, size, flags=0):
+        if size < len(self._data):
+            result = self._data[:size]
+            self._data = self._data[size:]
+            return result
+        else:
+            result = self._data
+            self._data = ""
+            return result
+
+class TestSocketDelay(TestCase):
+    def setUp(self):
+        TestCase.setUp(self)
+
+    def test_delay(self):
+        from bzrlib.transport.sftp import SocketDelay
+        sending = FakeSocket()
+        receiving = SocketDelay(sending, 0.1, 0)
+        # check that simulated time is charged only per round-trip:
+        t1 = SocketDelay.simulated_time
+        receiving.send("connect1")
+        self.assertEqual(sending.recv(1024), "connect1")
+        t2 = SocketDelay.simulated_time
+        self.assertAlmostEqual(t2 - t1, 0.1)
+        receiving.send("connect2")
+        self.assertEqual(sending.recv(1024), "connect2")
+        sending.send("hello")
+        self.assertEqual(receiving.recv(1024), "hello")
+        t3 = SocketDelay.simulated_time
+        self.assertAlmostEqual(t3 - t2, 0.1)
+        sending.send("hello")
+        self.assertEqual(receiving.recv(1024), "hello")
+        sending.send("hello")
+        self.assertEqual(receiving.recv(1024), "hello")
+        sending.send("hello")
+        self.assertEqual(receiving.recv(1024), "hello")
+        t4 = SocketDelay.simulated_time
+        self.assertAlmostEqual(t4, t3)
+
+    def test_bandwidth(self):
+        from bzrlib.transport.sftp import SocketDelay
+        sending = FakeSocket()
+        receiving = SocketDelay(sending, 0, 1)
+        # check that simulated time is charged only per round-trip:
+        t1 = SocketDelay.simulated_time
+        receiving.send("connect")
+        self.assertEqual(sending.recv(1024), "connect")
+        sending.send("a" * 100)
+        self.assertEqual(receiving.recv(1024), "a" * 100)
+        t2 = SocketDelay.simulated_time
+        self.assertAlmostEqual(t2 - t1, 100 + 7)
+

=== modified file 'bzrlib/transport/sftp.py'
--- bzrlib/transport/sftp.py	2006-07-20 23:55:43 +0000
+++ bzrlib/transport/sftp.py	2006-08-01 10:53:24 +0000
@@ -943,29 +943,44 @@
 class SocketDelay(object):
     """A socket decorator to make TCP appear slower.
 
-    This changes recv, send, and sendall to add a fixed latency to each 
-    python call. It will therefore behave differently to the real underlying
-    TCP stack which may only pay the latency price on each actual roundtrip
-    rather than each packet initiated. We can come closer to the real
-    behaviour by only charging latency for each roundtrip we detect - that is
-    when recv is called, toggle a flag, and insert latency again only when
-    send has been called in the middle. For now though, showing more latency
-    than we have is probably acceptable.
-
-    Not all methods are implemented, this is deliberate as this class is not
-    a replacement for the builtin sockets layer. fileno is not implemented to
-    prevent the proxy being bypassed.
+    This changes recv, send, and sendall to add a fixed latency to each python
+    call if a new roundtrip is detected. That is, when a recv is called and the
+    flag new_roundtrip is set, latency is charged. Every send and send_all
+    sets this flag.
+
+    In addition every send, sendall and recv sleeps a bit per character send to
+    simulate bandwidth.
+
+    The function used to sleep moves a counter forwards to not make the tests
+    slower. It could be made more clever, by adding the time that was passing
+    between sleep calls to the simulated time too.
+
+    Not all methods are implemented, this is deliberate as this class is not a
+    replacement for the builtin sockets layer. fileno is not implemented to
+    prevent the proxy being bypassed. 
     """
 
-    def __init__(self, sock, latency):
+    simulated_time = 0
+
+    def __init__(self, sock, latency, time_per_char=0.0001,
+                 really_sleep=False):
         self.sock = sock
         self.latency = latency
+        self.really_sleep = really_sleep
+        self.time_per_char = time_per_char
+        self.new_roundtrip = False
+
+    def sleep(self, s):
+        if self.really_sleep:
+            time.sleep(s)
+        SocketDelay.simulated_time += s
 
     def close(self):
         return self.sock.close()
 
     def dup(self):
-        return SocketDelay(self.sock.dup(), self.latency)
+        return SocketDelay(self.sock.dup(), self.latency, self.time_per_char,
+                           self._sleep)
 
     def getpeername(self, *args):
         return self.sock.getpeername(*args)
@@ -981,17 +996,26 @@
 
     def recv(self, *args):
         data = self.sock.recv(*args)
-        if data:
-            time.sleep(self.latency)
+        if data and self.new_roundtrip:
+            self.new_roundtrip = False
+            self.sleep(self.latency)
+            self.sleep(len(data) * self.time_per_char)
         return data
 
-    def sendall(self, *args):
-        time.sleep(self.latency)
-        return self.sock.sendall(*args)
+    def sendall(self, data, flags=0):
+        if not self.new_roundtrip:
+            self.new_roundtrip = True
+            self.sleep(self.latency)
+        self.sleep(len(data) * self.time_per_char)
+        return self.sock.sendall(data, flags)
 
-    def send(self, *args):
-        time.sleep(self.latency)
-        return self.sock.send(*args)
+    def send(self, data, flags=0):
+        if not self.new_roundtrip:
+            self.new_roundtrip = True
+            self.sleep(self.latency)
+        bytes_sent = self.sock.send(data, flags)
+        self.sleep(bytes_sent * self.time_per_char)
+        return bytes_sent
 
     def setblocking(self, *args):
         return self.sock.setblocking(*args)

